﻿using System;
using System.Linq;

namespace IdeaBlade.EntityModel {

  public abstract class CoroutineCompletedEventArgsWrapper<TOperation> 
    : CoroutineCompletedEventArgsWrapper where TOperation : BaseOperation {

    protected CoroutineCompletedEventArgsWrapper(CoroutineOperation triggerOperation)
      : base(triggerOperation) {
      TargetOperationName = typeof(TOperation).Name;
    }

    protected override void TriggerOperationCompletedHandler(object sender, CoroutineCompletedEventArgs e) {
      if (!e.CompletedSuccessfully) return;
      var targetOp = e.Notifications.LastOrDefault() as TOperation;
      if (null == targetOp ) {
        NoTargetEventArgsException = new InvalidOperationException(
          "Successful coroutine's LastCompletedOperation was not a " +
          TargetOperationName);
        return;
      }
      targetOp.Completed += (s, args) => TargetEventArgs = args;
    }
  }

  public abstract class CoroutineCompletedEventArgsWrapper : EventArgs, INotifyCompletedArgs {

    protected CoroutineCompletedEventArgsWrapper(CoroutineOperation triggerOperation) {

      TargetOperationName = "Operation";

      if (null == triggerOperation) throw new ArgumentNullException("triggerOperation");
      TriggerOperation = triggerOperation;
      TriggerOperation.Completed += TriggerOperationCompletedHandler;
    }

    protected abstract void TriggerOperationCompletedHandler(object sender, CoroutineCompletedEventArgs e);

    protected internal CoroutineOperation TriggerOperation { get; private set; }
    protected BaseOperation BaseOperation { get { return TriggerOperation; } }
    protected EventArgs TargetEventArgs { get; set; }

    public string TargetOperationName { get; protected internal set; }
    public bool Cancelled { get { return BaseOperation.Cancelled; } }
    public bool CompletedSuccessfully { get { return BaseOperation.CompletedSuccessfully; } }
    public bool CompletedSynchronously { get { return BaseOperation.CompletedSynchronously; } }
    public Exception Error { get { return BaseOperation.Error; } }
    public bool HasError { get { return BaseOperation.HasError; } }
    public bool IsCompleted { get { return BaseOperation.IsCompleted; } }
    public bool IsErrorHandled {
      get { return BaseOperation.IsErrorHandled; }
      set { BaseOperation.IsErrorHandled = value; }
    }
    public void MarkErrorAsHandled() { BaseOperation.MarkErrorAsHandled(); }
    public object UserState { get { return BaseOperation.UserState; } }

    protected Exception NoTargetEventArgsException { get; set; }

    protected void EnsureCompletedSuccessfully(string memberName) {
      memberName = memberName ?? "EventArg member";

      if (!TriggerOperation.IsCompleted) {
        throw new InvalidOperationException(
          TargetOperationName + " has not completed; cannot access " + memberName + ".");
      }

      if (!TriggerOperation.CompletedSuccessfully) {
        throw new InvalidOperationException(
          TargetOperationName + " did not complete successfully; cannot access " + memberName + ".");
      }

      if (null == TargetEventArgs) {
        
        if (null == NoTargetEventArgsException) {
        throw new InvalidOperationException(
          "Successful coroutine for "+TargetOperationName + " inexplicably lacks TargetEventArgs.");
        }
        throw NoTargetEventArgsException;

      }

    }
  }
}