// (c) Copyright IdeaBlade, Inc. and Microsoft Corporation
// Derived from Microsoft Silverlight User Test Framework// (c) Copyright Microsoft Corporation.
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.

using System;
using System.Threading;

namespace IdeaBlade.TestFramework {
  /// <summary>
  /// A test work item that is complete once the condition is asserted.
  /// </summary>
  public class ConditionalWorkItem : SynchronizedWorkItem {

    /// <summary>
    /// The conditional delegate.
    /// </summary>
    private Func<bool> _delegate;

    /// <summary>
    /// Construct a new conditional work item.
    /// </summary>
    /// <param name="conditionalMethod">Conditional delegate.</param>
    /// <param name="context">
    /// Optional <see cref="SynchronizationContext"/> in which callback should be executed.
    /// </param>
    public ConditionalWorkItem(Func<bool> conditionalMethod, SynchronizationContext context = null) {
      _delegate = conditionalMethod;
      SyncContext = context;
    }

    /// <summary>
    /// Invoke the condition, will continue invoking until 
    /// the condition is false.
    /// </summary>
    /// <returns>Completes the invocation once the condition is true.</returns>
    public override bool Invoke() {
      bool? isConditionTrue = false;
      Send(state => isConditionTrue = _delegate(), null);
      while (!isConditionTrue.HasValue) {
        Thread.Sleep(10); // wait for SyncContext to evaluate it.
      }

      if (isConditionTrue.Value) {
        WorkItemComplete();
      }
      return base.Invoke();
    }
  }
}