using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using IdeaBlade.EntityModel;
using Microsoft.Silverlight.Testing;

namespace IdeaBlade.TestFramework {

  // Silverlight specific portion of the DevForce Asynchronous test base class
  public partial class DevForceTest : SilverlightTest {

    /// <summary>
    /// Ensure an EntityManager is pinned to the thread 
    /// of the test's SynchronizationContext.
    /// </summary>
    /// <remarks>
    /// Needed in Desktop; not needed in SL tests where Manager would be created in the SyncContext.
    /// </remarks>
    protected void PinManagerToSyncContextThread(EntityManager manager) {}

    /// <summary>
    /// Get thread id of the <see cref="System.Threading.SynchronizationContext"/>
    /// used by this test which should be the same as the UI thread id
    /// where we are right now.
    /// </summary>
    public int SyncContextThreadId {
      get { return System.Threading.Thread.CurrentThread.ManagedThreadId; }
    }

    /// <summary>
    /// Wait for signal that the queued workitems are completed.
    /// </summary>
    /// <remarks>
    /// Does nothing in Silverlight but critical in Desktop tests.
    /// Defined here so that tests can be written to
    /// compile both in Silverlight and Desktop projects.
    /// </remarks>
    protected virtual void WaitForTasksToFinish() {}

    partial void SetTestClassBase() {
      // "WaitFor" extension methods refer to this instance on the SyncContext thread.
      TestClassBase = this;
    }

    // Haven't found the right answer to this yet.
    protected virtual void Log(string message) {
      Debug.WriteLine("*** [{0}] {1}", ++_logCount, message);
      _logMessages.Add(message);

      // Doesn't work. Failed binding (see Output Window)
      // Pinged Jeff Wilcox Aug 1. Check later.
      //this.UnitTestHarness.LogWriter.Information(message);
    }

    private int _logCount;
    private readonly ObservableCollection<String> _logMessages = new ObservableCollection<string>();
    /*
        private void CreateLogPanel() {
          var logPanel = new System.Windows.Controls.UserControl
                           {
                             Content = new System.Windows.Controls.ListBox { ItemsSource = _logMessages, }
                           };
          WaitFor.Event(logPanel, "Loaded");
          TestPanel.Children.Add(logPanel);
        }
    */

  }
}