﻿using System;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Diagnostics;
// ReSharper disable RedundantUsingDirective
using IdeaBlade.Core;
using IdeaBlade.Core.ComponentModel.DataAnnotations;  //needed in SL
// ReSharper restore RedundantUsingDirective
using IdeaBlade.Extensions;
using IdeaBlade.Validation;

// ReSharper disable RedundantNameQualifier
// ReSharper disable InconsistentNaming

namespace SimpleNorthwindModel
{
    [MetadataType(typeof(Customer.CustomerMetadata))] 
    [DebuggerDisplay("{ToString()}")]
    public partial class Customer
    {
        public Customer()
        {
            CustomerID = SystemGuid.NewGuid();
        }

        [Display(Name = "Original", AutoGenerateField = true, Order = 3)]
        public bool IsOriginal
        {
            get { return CustomerID_OLD != String.Empty; }
        }

        public bool CanDelete { get { return !IsOriginal; }}

        // Enforce integrity of order before adding it
        public void AddOrder(Order order)
        {
            // To Do: something real
            order.Customer = this;
        }

        // Enforce rules for removing an order
        public void RemoveOrder(Order order)
        {
            throw new InvalidOperationException("Can't remove an order");
        }

        public override string ToString()
        {
            return String.Format("{0}({1})", CompanyName, CustomerID);
        }

        // Trim spaces from Company name
        [BeforeSet(EntityPropertyNames.CompanyName)]
        public void TrimNameBeforeSet(
            PropertyInterceptorArgs<Customer, String> args)
        {
            if (args.Value != null) args.Value = args.Value.Trim();
        }

        #region Customer Metadata

        public static class CustomerMetadata
        {
            [Display(Name = "ID", AutoGenerateField = true, Order = 0)]
            [Editable(false)]
            public static Guid CustomerID;

            [Display(Name = "Name", AutoGenerateField = true, Order = 1)]
            [DoNothingCustom]
            public static string CompanyName;

            // Country is nullable in database but we want to require it
            [RequiredValueVerifier]
            public static string Country;

            [Bindable(true, BindingDirection.OneWay)]
            [Editable(false)]
            [Display(Name = "CustomerID_OLD", AutoGenerateField = false)]
            public static Guid CustomerID_OLD;
        }

        #endregion
    }

    public class DoNothingCustomAttribute : Attribute {}
}
