﻿using System;
using IdeaBlade.Extensions;
using Microsoft.Silverlight.Testing;
using Microsoft.VisualStudio.TestTools.UnitTesting;

// ReSharper disable InconsistentNaming

namespace WorkingWithEntities.SystemUtilityTests
{
    /// <summary>
    /// Demonstrate SystemTime utility class
    /// </summary>
    [TestClass]
    public class When_using_SystemTime
    {
        [TestMethod]
        [Tag("SystemTime")]
        public void can_override_NowFn()
        {
            var expect = DateTime.Now;

            System.Threading.Thread.Sleep(1);
            Assert.AreNotEqual(expect, DateTime.Now, "NowFn not replaced");

            SystemTime.NowFn = () => expect;
            Assert.AreEqual(expect, SystemTime.Now(), "Not same 1st time");
            System.Threading.Thread.Sleep(1);
            Assert.AreEqual(expect, SystemTime.Now(), "Not same 2nd time");

            SystemTime.ResetNowFn();
            Assert.AreNotEqual(expect, SystemTime.Now(), "NowFn restore failed");
        }
    }

  /// <summary>
  /// Demonstrate SystemGuid utility class
  /// </summary>
  [TestClass]
  public class When_using_SystemGuid 
  {
    [TestMethod]
    [Tag("SystemGuid")]
    public void can_override_NewGuidFn()
    {
        var expect = Guid.NewGuid();
        Assert.AreNotEqual(expect, Guid.NewGuid(), "NewGuidFn not replaced");

        SystemGuid.NewGuidFn = () => expect;
        Assert.AreEqual(expect, SystemGuid.NewGuid(), "Not same 1st time");
        Assert.AreEqual(expect, SystemGuid.NewGuid(), "Not same 2nd time");

        SystemGuid.ResetNewGuidFn();
        Assert.AreNotEqual(expect, SystemGuid.NewGuid(), "NewGuidFn restore failed");

    }
  }
}