Imports System.Collections.ObjectModel
Imports IdeaBlade.EntityModel
Imports Microsoft.Silverlight.Testing

Namespace IdeaBlade.TestFramework

  ' Silverlight specific portion of the DevForce Asynchronous test base class
  Partial Public Class DevForceTest
	  Inherits SilverlightTest

	''' <summary>
	''' Ensure an EntityManager is pinned to the thread 
	''' of the test's SynchronizationContext.
	''' </summary>
	''' <remarks>
	''' Needed in Desktop; not needed in SL tests where Manager would be created in the SyncContext.
	''' </remarks>
	Protected Sub PinManagerToSyncContextThread(ByVal manager As EntityManager)
	End Sub

	''' <summary>
	''' Get thread id of the <see cref="System.Threading.SynchronizationContext"/>
	''' used by this test which should be the same as the UI thread id
	''' where we are right now.
	''' </summary>
	Public ReadOnly Property SyncContextThreadId() As Integer
	  Get
		  Return System.Threading.Thread.CurrentThread.ManagedThreadId
	  End Get
	End Property

	''' <summary>
	''' Wait for signal that the queued workitems are completed.
	''' </summary>
	''' <remarks>
	''' Does nothing in Silverlight but critical in Desktop tests.
	''' Defined here so that tests can be written to
	''' compile both in Silverlight and Desktop projects.
	''' </remarks>
	Protected Overridable Sub WaitForTasksToFinish()
	End Sub

	Private Sub SetTestClassBase()
	  ' "WaitFor" extension methods refer to this instance on the SyncContext thread.
	  TestClassBase = Me
	End Sub

	' Haven't found the right answer to this yet.
	Protected Overridable Sub Log(ByVal message As String)
	  _logCount += 1
	  Debug.WriteLine("*** [{0}] {1}", _logCount, message)
	  _logMessages.Add(message)

	  ' Doesn't work. Failed binding (see Output Window)
	  ' Pinged Jeff Wilcox Aug 1. Check later.
	  'this.UnitTestHarness.LogWriter.Information(message);
	End Sub

	Private _logCount As Integer
	Private ReadOnly _logMessages As New ObservableCollection(Of String)()
'    
'        private void CreateLogPanel() {
'          var logPanel = new System.Windows.Controls.UserControl
'                           {
'                             Content = new System.Windows.Controls.ListBox { ItemsSource = _logMessages, }
'                           };
'          WaitFor.Event(logPanel, "Loaded");
'          TestPanel.Children.Add(logPanel);
'        }
'    

  End Class
End Namespace