Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.ComponentModel.DataAnnotations
' ReSharper disable RedundantUsingDirective
Imports IdeaBlade.Core.ComponentModel.DataAnnotations 'needed in SL
' ReSharper restore RedundantUsingDirective
Imports IdeaBlade.EntityModel
Imports IdeaBlade.Validation

' ReSharper disable RedundantNameQualifier

<MetadataType(GetType(Employee.EmployeeMetadata))>
Partial Public Class Employee ' Must specify "Employee." no matter what R# says

    <Display(Name:="Full Name", AutoGenerateField:=True)>
    Public ReadOnly Property FullName() As String
        Get
            Return If(EntityAspect.IsNullEntity, "[None]", FirstName & " " & LastName)
        End Get
    End Property

    Public Overrides Function ToString() As String ' hack
        Return FullName
    End Function

    ''' <summary>Gets Orders placed this year (which is always 1998).</summary>
    ''' <remarks>
    ''' An example of a filtered navigation property: one that actively
    ''' filters by some selection criteria.
    ''' It returns a "live" collection meaning that the list adjusts
    ''' automatically as books that pass the selection criteria appear
    ''' and disappear from the entity cache.
    ''' See it tested in CacheOnlyQueryTests.
    ''' <para>
    ''' ToDo: Should be ReadOnly; vulnerable to direct Add and Remove
    ''' that could violate list membership rules.
    ''' </para>
    ''' </remarks>
    <Bindable(False), Display(Name:="CY Orders", AutoGenerateField:=False)>
    Public ReadOnly Property CurrentYearOrders() As ObservableCollection(Of Order)
        Get
            Return If(_cyOrders, CreateCyOrders())
        End Get
    End Property

    Private Function CreateCyOrders() As ObservableCollection(Of Order)
        _cyOrders = New ObservableCollection(Of Order)()
        Dim orderElm = New EntityListManager(Of Order)(EntityAspect.EntityManager, Function(o) o.EmployeeID = EmployeeID AndAlso o.OrderDate.HasValue AndAlso o.OrderDate.Value.Year = 1998, New EntityProperty() {Order.PropertyMetadata.EmployeeID, Order.PropertyMetadata.OrderDate}, _cyOrders, True)

        Return _cyOrders


        ' THIS DOESN'T WORK
        'return new RelatedEntityList<Order>(
        '   this.Orders.Where(o => o.OrderDate != null && o.OrderDate.Value.Year == 1998),
        '   PropertyMetadata.Orders.GetEntityReference(this)
        ');
    End Function

    Private _cyOrders As ObservableCollection(Of Order)

#Region "Employee Metadata"

    Public NotInheritable Class EmployeeMetadata
        ' City is nullable in database but we want to require it
        <RequiredValueVerifier()>
        Public Shared City As String

        <DateTimeRangeVerifier(MinValue:="1/1/2000", MaxValue:="12/31/2010")>
        Public Shared HireDate? As Date

        <Display(Name:="Home #", AutoGenerateField:=True), RegexVerifier("Home phone", "USPhone")>
        Public Shared HomePhone As String
    End Class
#End Region
End Class
