﻿Imports System.ComponentModel ' keep for SL
Imports IdeaBlade.EntityModel
Imports Microsoft.Silverlight.Testing
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports SimpleNorthwindModel

' ReSharper disable InconsistentNaming
' ReSharper disable RedundantDefaultFieldInitializer
' ReSharper disable RedundantArgumentName

Namespace WorkingWithEntities.EntityVersionTests

	<TestClass>
	Public Class When_start_with_unchanged_test_customer
		Inherits EntityVersionTests
		<TestInitialize>
		Public Sub TestInitialize()
			manager = New EntityManager(shouldConnect:= False)
			testCustomer = New Customer With {.CustomerID = Guid.NewGuid(), .CompanyName = testCompanyName}
			AttachTestCustomerAsUnchanged()
		End Sub

		<TestMethod, Tag("Version")>
		Public Sub then_testCustomer_isUnchanged_at_start()
			Assert.IsTrue(TestCustomerEntityState.IsUnchanged())
		End Sub

		<TestMethod, Tag("Version")>
		Public Sub then_orig_equals_cur_at_start()
			Assert.AreEqual(testCompanyName, GetOriginalName(), "testCompanyName should match orig")
		End Sub

		<TestMethod, Tag("Version")>
		Public Sub can_access_property_if_detached()
			testCustomer.EntityAspect.RemoveFromManager()
			Assert.IsTrue(TestCustomerEntityState.IsDetached())
			Assert.AreEqual(testCompanyName, testCustomer.CompanyName)
		End Sub

		<TestMethod, Tag("Version")>
		Public Sub then_orig_is_defined_after_detach()
			testCustomer.EntityAspect.RemoveFromManager()
			Assert.IsTrue(TestCustomerEntityState.IsDetached(), "Should be detached")

			Assert.AreEqual(testCompanyName, GetOriginalName(), "testCompanyName should match orig")
		End Sub

		<TestMethod, Tag("Version")>
		Public Sub then_cur_differs_from_orig_if_modified()
			Const newName As String = "New Name"
			testCustomer.CompanyName = newName
			Assert.IsTrue(TestCustomerEntityState.IsModified(), "Should be modified")

			Assert.AreEqual(GetName(), GetCurrentName(), "property should match cur")

			Assert.AreEqual(newName, GetCurrentName(), "newName should match cur")

			Assert.AreEqual(testCompanyName, GetOriginalName(), "testCompanyName should match orig")
		End Sub

		<TestMethod, Tag("Version")>
		Public Sub then_cur_equals_orig_after_undo()
			Dim origName = testCustomer.CompanyName
			Const newName As String = "New Name"

			testCustomer.CompanyName = newName

			Assert.IsTrue(TestCustomerEntityState.IsModified(), "Should be modified after setting name")

			testCustomer.EntityAspect.RejectChanges()

			Assert.IsTrue(TestCustomerEntityState.IsUnchanged(), "Should be unchanged after undo")

			Assert.AreEqual(origName, GetName(), "property should match orig")

			Assert.AreEqual(origName, GetCurrentName(), "origName should match cur")

			Assert.AreEqual(origName, GetOriginalName(), "origName should match orig")
		End Sub

		<TestMethod, Tag("Version")>
		Public Sub then_default_equals_orig_after_delete()
			Dim origName = testCustomer.CompanyName
			Const newName As String = "New Name"

			testCustomer.CompanyName = newName

			Assert.IsTrue(TestCustomerEntityState.IsModified(), "Should be modified after setting name")

			testCustomer.EntityAspect.Delete()

			Assert.IsTrue(TestCustomerEntityState.IsDeleted(), "Should be deleted after delete")

			' Default maps to Original Version
			Assert.AreEqual(origName, GetDefaultName(), "origName should match default")

			Assert.AreEqual(origName, GetName(), "origName should match property")

			' Cur version is untouched
			Assert.AreEqual(newName, GetCurrentName(), "newName should match cur")

			Assert.AreEqual(origName, GetOriginalName(), "origName should match orig")
		End Sub

		<TestMethod, Tag("Version")>
		Public Sub then_put_it_through_the_wringer()
			' testCustomer is unchanged as we begin

			Dim origName = testCustomer.CompanyName
			Const newName As String = "New Name"
			Const proposedName As String = "Proposed Name"

			' Change the current value.
			testCustomer.CompanyName = newName

			Assert.IsTrue(TestCustomerEntityState.IsModified(), "Should be modified")

			Assert.AreEqual(newName, GetName(), "newName should match the property getter value before edit")

			Assert.AreEqual(newName, GetDefaultName(), "newName should match the default value before edit")

			' before edit session, proposed is current
			Assert.AreEqual(newName, GetProposedName(), "newName should match the proposed value before edit")

			Assert.AreEqual(newName, GetCurrentName(), "newName should match the current value before edit")

			Assert.AreEqual(origName, GetOriginalName(), "origName should match the original value before edit")

			' Open IEditableObject editing session
			Dim ieo = TryCast(testCustomer, IEditableObject)
			ieo.BeginEdit()

			' set tentative value within editing session
			testCustomer.CompanyName = proposedName

			Assert.AreEqual(proposedName, GetName(), "proposedName should match the property getter value during edit")

			Assert.AreEqual(proposedName, GetDefaultName(), "proposedName should match the default value during edit")

			Assert.AreEqual(proposedName, GetProposedName(), "proposedName should match the proposed value during edit")

			Assert.AreNotEqual(proposedName, GetCurrentName(), "proposedName should not match the current value during edit")

			Assert.AreEqual(origName, GetOriginalName(), "origName should match the original value during edit")

			' Cancel IEditableObject editing session
			ieo.CancelEdit()

			' After cancel, current should revert

			Assert.AreEqual(newName, GetName(), "newName should match the property getter value after cancel")

			Assert.AreEqual(newName, GetDefaultName(), "newName should match the default value after cancel")

			' Notice that proposed reverted too
			Assert.AreEqual(newName, GetProposedName(), "newName should match the proposed value after cancel")

			Assert.AreEqual(newName, GetCurrentName(), "newName should match the current value after cancel")

			Assert.AreEqual(origName, GetOriginalName(), "origName should match the original value after cancel")

			' Now detach 
			testCustomer.EntityAspect.RemoveFromManager()

			Assert.AreEqual(newName, GetName(), "newName should match the property getter value of detached")

			Assert.AreEqual(newName, GetDefaultName(), "newName should match the default value of detached")

			Assert.AreEqual(newName, GetProposedName(), "newName should match the proposed value of detached")

			Assert.AreEqual(newName, GetCurrentName(), "newName should match the current value of detached")

			Assert.AreEqual(origName, GetOriginalName(), "origName should match the original value of detached")

			' Now re-attach AS MODIFIED
			manager.AttachEntity(testCustomer, EntityState.Modified)
			Assert.IsTrue(TestCustomerEntityState.IsModified(), "Should be modified after re-attachment as modified")

			Assert.AreEqual(newName, GetName(), "newName should match the property getter value of re-attached")

			Assert.AreEqual(newName, GetDefaultName(), "newName should match the default value of re-attached")

			Assert.AreEqual(newName, GetProposedName(), "newName should match the proposed value of re-attached")

			Assert.AreEqual(newName, GetCurrentName(), "newName should match the current value of re-attached")

			' N.B. The original is restored when re-attached
			Assert.AreEqual(origName, GetOriginalName(), "origName should match the original value of re-attached")
		End Sub

	End Class

	<TestClass>
	Public Class When_start_with_new_test_customer
		Inherits EntityVersionTests
		' Added entities don't have original
		' If you ask for the original of an added entity
		' you get the "default"

		<TestInitialize>
		Public Sub TestInitialize()
			manager = New EntityManager(shouldConnect:= False)
			testCustomer = New Customer With {.CompanyName = testCompanyName, .Country = "Freedonia"}
			manager.AddEntity(testCustomer)
		End Sub

		<TestMethod, Tag("Version")>
		Public Sub then_testCustomer_isAdded_at_start()
			Assert.IsTrue(TestCustomerEntityState.IsAdded())
		End Sub

		<TestMethod, Tag("Version")>
		Public Sub then_default_equals_cur_after_delete()
			Dim initialName = GetName()

			Assert.IsTrue(TestCustomerEntityState.IsAdded(), "Should be added at start")

			testCustomer.EntityAspect.Delete()

			' New entities are detached after they are deleted
			' No point in saving them as there should be no
			' corresponding database object to delete.
			Assert.IsTrue(TestCustomerEntityState.IsDetached(), "Should be detached after delete")

			' Default maps to the pre-delete current value
			Assert.AreEqual(initialName, GetDefaultName(), "initialName should match default")

			Assert.AreEqual(initialName, GetName(), "initialName should match property")

			Assert.AreEqual(initialName, GetCurrentName(), "initialName should match cur")

		End Sub

	End Class

	#Region "EntityVersionTests base class"

	Public MustInherit Class EntityVersionTests

		Protected testCustomer As Customer
		Protected Const testCompanyName As String = "Test Company"
		Protected manager As EntityManager

		Protected Sub AttachTestCustomerAsUnchanged()
			manager.AttachEntity(testCustomer)
			testCustomer.EntityAspect.SetModified()
			testCustomer.EntityAspect.AcceptChanges()
		End Sub

		Protected ReadOnly Property TestCustomerEntityState() As EntityState
			Get
				Return testCustomer.EntityAspect.EntityState
			End Get
		End Property

		#Region "GetName in all versions"

		Protected Function GetName() As String
			Return testCustomer.CompanyName
		End Function

		Protected Function GetCurrentName() As String
			Return CStr(testCustomer.EntityAspect.GetValue("CompanyName", EntityVersion.Current))
		End Function

		Protected Function GetOriginalName() As String
			Return CStr(testCustomer.EntityAspect.GetValue("CompanyName", EntityVersion.Original))
		End Function

		Protected Function GetProposedName() As String
			Return CStr(testCustomer.EntityAspect.GetValue("CompanyName", EntityVersion.Proposed))
		End Function

		Protected Function GetDefaultName() As String
			Return CStr(testCustomer.EntityAspect.GetValue("CompanyName", EntityVersion.Default))
		End Function

		#End Region      
	End Class

	#End Region

End Namespace