﻿Imports System.Text.RegularExpressions
Imports IdeaBlade.Core
Imports IdeaBlade.EntityModel
Imports Microsoft.Silverlight.Testing
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports SimpleNorthwindModel

' ReSharper disable InconsistentNaming
' ReSharper disable RedundantDefaultFieldInitializer
' ReSharper disable RedundantArgumentName

Namespace WorkingWithEntities.PropertyInterception

	<TestClass>
	Public Class When_changing_customer_in_cache
		<TestInitialize>
		Public Sub TestInitialize()
			manager = New EntityManager(shouldConnect:= False)
			testCustomer = New Customer With {.CompanyName = "Acme", .Country = "Freedonia"}
			manager.AddEntity(testCustomer)
		End Sub

		''' <summary>Test TrimCompanyName attributed interceptor</summary>
		<TestMethod, Tag("Intercept")>
		Public Sub interceptor_trims_CompanyName()
			Const newRawName As String = "   spaces  all   over  "

			testCustomer.CompanyName = newRawName

			Dim newTrimmedName = newRawName.Trim()
			Assert.AreEqual(newTrimmedName, testCustomer.CompanyName)
		End Sub

		''' <summary>Test TrimCompanyName attributed interceptor with null</summary>
		<TestMethod, Tag("Intercept")>
		Public Sub interceptor_accepts_null_CompanyName()
			testCustomer.CompanyName = Nothing ' interceptor doesn't throw
			Assert.IsNull(testCustomer.CompanyName)
			'N.B.: It is invalid but it didn't throw
		End Sub

		''' <summary>Test ContactName w/o runtime interceptor</summary>
		<TestMethod, Tag("Intercept")>
		Public Sub without_interceptor_ContactName_keeps_whitespace()
			Const newRawContactName As String = "   spaces  all   over  "

			' Ensure no property interceptor set for this property.
			RemoveContactNameInterceptor()

			testCustomer.ContactName = newRawContactName

			Assert.IsTrue(testCustomer.ContactName.Contains(" "), "Space found in ContactName")
		End Sub

		''' <summary>Test null ContactName with runtime interceptor</summary>
		<TestMethod, Tag("Intercept")>
		Public Sub interceptor_accepts_null_ContactName()
			AddContactNameInterceptor()

			testCustomer.ContactName = Nothing
			Assert.IsNull(testCustomer.ContactName)
		End Sub

		''' <summary>Test space-filled ContactName with runtime interceptor</summary>
		<TestMethod, Tag("Intercept")>
		Public Sub interceptor_strips_ContactName_whitespace()
			AddContactNameInterceptor()

			Const newRawContactName As String = "   spaces  all   over  "
			testCustomer.ContactName = newRawContactName

			Assert.IsFalse(testCustomer.ContactName.Contains(" "), "Space found in ContactName")
		End Sub

		Protected manager As EntityManager
		Protected testCustomer As Customer

		Private Shared Sub AddContactNameInterceptor()
		  If PropertyInterceptorManager.CurrentInstance.GetActions(Of DataEntityPropertySetInterceptorArgs(Of Customer, String))(GetType(Customer), Customer.EntityPropertyNames.ContactName, PropertyInterceptorMode.BeforeSet).Count > 0 Then
			  Return
		  End If

      Dim act = New PropertyInterceptorAction(Of DataEntityPropertySetInterceptorArgs(Of Customer, String))(GetType(Customer), Customer.EntityPropertyNames.ContactName, PropertyInterceptorMode.BeforeSet, Sub(args)
                                                                                                                                                                                                              If Nothing Is args.Value Then
                                                                                                                                                                                                                Return
                                                                                                                                                                                                              End If
                                                                                                                                                                                                              args.Value = Regex.Replace(args.Value, "\s", "")
                                                                                                                                                                                                            End Sub)

		  PropertyInterceptorManager.CurrentInstance.AddAction(act)
		End Sub

		Private Shared Sub RemoveContactNameInterceptor()
		  Dim actions = PropertyInterceptorManager.CurrentInstance.GetActions(Of DataEntityPropertySetInterceptorArgs(Of Customer, String))(GetType(Customer), Customer.EntityPropertyNames.ContactName, PropertyInterceptorMode.BeforeSet).ToList()
		  actions.ForEach(Function(a) PropertyInterceptorManager.CurrentInstance.RemoveAction(a))
		End Sub
	End Class
End Namespace