﻿Imports System.ComponentModel
Imports IdeaBlade.Core.ComponentModel
Imports IdeaBlade.EntityModel
Imports Microsoft.Silverlight.Testing
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports SimpleNorthwindModel

' ReSharper disable InconsistentNaming
' ReSharper disable RedundantDefaultFieldInitializer
' ReSharper disable RedundantArgumentName

Namespace WorkingWithEntities.Validation

	<TestClass>
	Public Class When_validating_customer_in_cache
		<TestInitialize>
		Public Sub TestInitialize()
			manager = New EntityManager(shouldConnect:= False)
			testCustomer = New Customer With {.CompanyName = "Acme", .Country = "Freedonia"}
			manager.AddEntity(testCustomer)
		End Sub

		<TestMethod, Tag("Validate")>
		Public Sub testCustomer_is_error_free_at_start()
			Assert.IsFalse(testCustIndei.HasErrors, "Should be error free at start")
		End Sub

		''' <summary>Test application of <see cref="NotEmptyGuidVerifier"/></summary>
		''' <remarks>
		''' Get an error because of custom validation rule
		''' <see cref="NotEmptyGuidVerifier"/> added in
		''' <see cref="NorthwindModelVerifierProvider"/>.
		''' </remarks>
		<TestMethod, Tag("Validate")>
		Public Sub has_error_when_set_CustomerID_empty()
			testCustomer.CustomerID = Guid.Empty
			Assert.IsTrue(testCustIndei.HasErrors, "Should have error when ID is Guid.Empty")
		End Sub

		''' <summary>Test verifier attribute added to Country in partial class</summary>
		<TestMethod, Tag("Validate")>
		Public Sub has_error_when_set_Country_to_empty_string()
			testCustomer.Country = String.Empty
			Assert.IsTrue(testCustIndei.HasErrors, "Should have error when Country is String.Empty")
		End Sub

		Protected manager As EntityManager
		Protected testCustomer As Customer
		Protected ReadOnly Property testCustIdei() As IDataErrorInfo
			Get
				Return testCustomer
			End Get
		End Property
		Protected ReadOnly Property testCustIndei() As INotifyDataErrorInfo
			Get
				Return testCustomer
			End Get
		End Property

	End Class
End Namespace